@@ -1,57 +1,45 @@
-# ##### BEGIN GPL LICENSE BLOCK #####
-#
-#  This program is free software; you can redistribute it and/or
-#  modify it under the terms of the GNU General Public License
-#  as published by the Free Software Foundation; either version 2
-#  of the License, or (at your option) any later version.
-#
-#  This program is distributed in the hope that it will be useful,
-#  but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#  GNU General Public License for more details.
-#
-#  You should have received a copy of the GNU General Public License
-#  along with this program; if not, write to the Free Software Foundation,
-#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
-#
-# ##### END GPL LICENSE BLOCK #####
-
-# <pep8-80 compliant>
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 bl_info = {
     "name": "IvyGen",
     "author": "testscreenings, PKHG, TrumanBlending",
-    "version": (0, 1, 2),
-    "blender": (2, 59, 0),
-    "location": "View3D > Add > Curve",
+    "version": (0, 1, 5),
+    "blender": (2, 80, 0),
+    "location": "View3D > Sidebar > Ivy Generator (Create Tab)",
     "description": "Adds generated ivy to a mesh object starting "
                    "at the 3D cursor",
     "warning": "",
-    "wiki_url": "https://wiki.blender.org/index.php/Extensions:2.6/Py/"
-                "Scripts/Curve/Ivy_Gen",
+    "doc_url": "{BLENDER_MANUAL_URL}/addons/add_curve/ivy_gen.html",
     "category": "Add Curve",
 }
 
 
 import bpy
+from bpy.types import (
+    Operator,
+    Panel,
+    PropertyGroup,
+)
 from bpy.props import (
-        FloatProperty,
-        IntProperty,
-        BoolProperty,
-        )
+    BoolProperty,
+    FloatProperty,
+    IntProperty,
+    PointerProperty,
+)
+from mathutils.bvhtree import BVHTree
 from mathutils import (
-        Vector,
-        Matrix,
-        )
+    Vector,
+    Matrix,
+)
 from collections import deque
 from math import (
-        pow, cos,
-        pi, atan2,
-        )
+    pow, cos,
+    pi, atan2,
+)
 from random import (
-        random as rand_val,
-        seed as rand_seed,
-        )
+    random as rand_val,
+    seed as rand_seed,
+)
 import time
 
 
@@ -61,7 +49,7 @@ def createIvyGeometry(IVY, growLeaves):
     # local_ivyBranchSize = IVY.ivyBranchSize  # * radius * IVY.ivySize
     gaussWeight = (1.0, 2.0, 4.0, 7.0, 9.0, 10.0, 9.0, 7.0, 4.0, 2.0, 1.0)
 
-    # Create a new curve and intialise it
+    # Create a new curve and initialise it
     curve = bpy.data.curves.new("IVY", type='CURVE')
     curve.dimensions = '3D'
     curve.bevel_depth = 1
@@ -187,20 +175,20 @@ def createIvyGeometry(IVY, growLeaves):
 
     # Add the object and link to scene
     newCurve = bpy.data.objects.new("IVY_Curve", curve)
-    bpy.context.scene.objects.link(newCurve)
+    bpy.context.collection.objects.link(newCurve)
 
     if growLeaves:
         faceList = [[4 * i + l for l in range(4)] for i in
-                                                     range(len(vertList) // 4)]
+                    range(len(vertList) // 4)]
 
         # Generate the new leaf mesh and link
         me = bpy.data.meshes.new('IvyLeaf')
         me.from_pydata(vertList, [], faceList)
         me.update(calc_edges=True)
         ob = bpy.data.objects.new('IvyLeaf', me)
-        bpy.context.scene.objects.link(ob)
+        bpy.context.collection.objects.link(ob)
 
-        me.uv_textures.new("Leaves")
+        me.uv_layers.new(name="Leaves")
 
         # Set the uv texture coords
         # TODO, this is non-functional, default uvs are ok?
@@ -212,24 +200,6 @@ def createIvyGeometry(IVY, growLeaves):
         ob.parent = newCurve
 
 
-'''
-def computeBoundingSphere(ob):
-    # Get the mesh data
-    me = ob.data
-    # Intialise the center
-    center = Vector((0.0, 0.0, 0.0))
-    # Add all vertex coords
-    for v in me.vertices:
-        center += v.co
-    # Average over all verts
-    center /= len(me.vertices)
-    # Create the iterator and find its max
-    length_iter = ((center - v.co).length for v in me.vertices)
-    radius = max(length_iter)
-    return radius
-'''
-
-
 class IvyNode:
     """ The basic class used for each point on the ivy which is grown."""
     __slots__ = ('pos', 'primaryDir', 'adhesionVector', 'adhesionLength',
@@ -289,7 +259,7 @@ class Ivy:
         self.maxAdhesionDistance = maxAdhesionDistance
         self.maxLength = 0.0
 
-        # Normalize all the weights only on intialisation
+        # Normalize all the weights only on initialisation
         sums = self.primaryWeight + self.randomWeight + self.adhesionWeight
         self.primaryWeight /= sums
         self.randomWeight /= sums
@@ -304,7 +274,7 @@ class Ivy:
         tmpRoot.ivyNodes.append(tmpIvy)
         self.ivyRoots.append(tmpRoot)
 
-    def grow(self, ob):
+    def grow(self, ob, bvhtree):
         # Determine the local sizes
         # local_ivySize = self.ivySize  # * radius
         # local_maxFloatLength = self.maxFloatLength  # * radius
@@ -331,8 +301,8 @@ class Ivy:
             randomVector.normalize()
 
             # Calculate the adhesion vector
-            adhesionVector = adhesion(prevIvy.pos, ob,
-                                                      self.maxAdhesionDistance)
+            adhesionVector = adhesion(
+                    prevIvy.pos, bvhtree, self.maxAdhesionDistance)
 
             # Calculate the growing vector
             growVector = self.ivySize * (primaryVector * self.primaryWeight +
@@ -349,7 +319,7 @@ class Ivy:
             newPos = prevIvy.pos + growVector + gravityVector
 
             # Check for collisions with the object
-            climbing = collision(ob, prevIvy.pos, newPos)
+            climbing, newPos = collision(bvhtree, prevIvy.pos, newPos)
 
             # Update the growing vector for any collisions
             growVector = newPos - prevIvy.pos - gravityVector
@@ -408,17 +378,13 @@ class Ivy:
                         return
 
 
-def adhesion(loc, ob, max_l):
-    # Get transfor vector and transformed loc
-    tran_mat = ob.matrix_world.inverted()
-    tran_loc = tran_mat * loc
-
+def adhesion(loc, bvhtree, max_l):
     # Compute the adhesion vector by finding the nearest point
-    nearest_result = ob.closest_point_on_mesh(tran_loc, max_l)
+    nearest_location, *_ = bvhtree.find_nearest(loc, max_l)
     adhesion_vector = Vector((0.0, 0.0, 0.0))
-    if nearest_result[0]:
+    if nearest_location is not None:
         # Compute the distance to the nearest point
-        adhesion_vector = ob.matrix_world * nearest_result[1] - loc
+        adhesion_vector = nearest_location - loc
         distance = adhesion_vector.length
         # If it's less than the maximum allowed and not 0, continue
         if distance:
@@ -429,31 +395,39 @@ def adhesion(loc, ob, max_l):
     return adhesion_vector
 
 
-def collision(ob, pos, new_pos):
+def collision(bvhtree, pos, new_pos):
     # Check for collision with the object
     climbing = False
 
-    # Transform vecs
-    tran_mat = ob.matrix_world.inverted()
-    tran_pos = tran_mat * pos
-    tran_new_pos = tran_mat * new_pos
-    tran_dir = tran_new_pos - tran_pos
+    corrected_new_pos = new_pos
+    direction = new_pos - pos
 
-    ray_result = ob.ray_cast(tran_pos, tran_dir, tran_dir.length)
+    hit_location, hit_normal, *_ = bvhtree.ray_cast(pos, direction, direction.length)
     # If there's a collision we need to check it
-    if ray_result[0]:
+    if hit_location is not None:
         # Check whether the collision is going into the object
-        if tran_dir.dot(ray_result[2]) < 0.0:
-            # Find projection of the piont onto the plane
-            p0 = tran_new_pos - (tran_new_pos -
-                                          ray_result[1]).project(ray_result[2])
-            # Reflect in the plane
-            tran_new_pos += 2 * (p0 - tran_new_pos)
-            new_pos *= 0
-            new_pos += ob.matrix_world * tran_new_pos
-            climbing = True
-    return climbing
+        if direction.dot(hit_normal) < 0.0:
+            reflected_direction = (new_pos - hit_location).reflect(hit_normal)
 
+            corrected_new_pos = hit_location + reflected_direction
+            climbing = True
+
+    return climbing, corrected_new_pos
+
+
+def bvhtree_from_object(ob):
+    import bmesh
+    bm = bmesh.new()
+
+    depsgraph = bpy.context.evaluated_depsgraph_get()
+    ob_eval = ob.evaluated_get(depsgraph)
+    mesh = ob_eval.to_mesh()
+    bm.from_mesh(mesh)
+    bm.transform(ob.matrix_world)
+
+    bvhtree = BVHTree.FromBMesh(bm)
+    ob_eval.to_mesh_clear()
+    return bvhtree
 
 def check_mesh_faces(ob):
     me = ob.data
@@ -463,127 +437,22 @@ def check_mesh_faces(ob):
     return False
 
 
-class IvyGen(bpy.types.Operator):
+class IvyGen(Operator):
     bl_idname = "curve.ivy_gen"
     bl_label = "IvyGen"
     bl_description = "Generate Ivy on an Mesh Object"
     bl_options = {'REGISTER', 'UNDO'}
 
-    maxIvyLength = FloatProperty(
-            name="Max Ivy Length",
-            description="Maximum ivy length in Blender Units",
-            default=1.0,
-            min=0.0,
-            soft_max=3.0,
-            subtype='DISTANCE',
-            unit='LENGTH'
-            )
-    primaryWeight = FloatProperty(
-            name="Primary Weight",
-            description="Weighting given to the current direction",
-            default=0.5,
-            min=0.0,
-            soft_max=1.0
-            )
-    randomWeight = FloatProperty(
-            name="Random Weight",
-            description="Weighting given to the random direction",
-            default=0.2,
-            min=0.0,
-            soft_max=1.0
-            )
-    gravityWeight = FloatProperty(
-            name="Gravity Weight",
-            description="Weighting given to the gravity direction",
-            default=1.0,
-            min=0.0,
-            soft_max=1.0
-            )
-    adhesionWeight = FloatProperty(
-            name="Adhesion Weight",
-            description="Weighting given to the adhesion direction",
-            default=0.1,
-            min=0.0,
-            soft_max=1.0
-            )
-    branchingProbability = FloatProperty(
-            name="Branching Probability",
-            description="Probability of a new branch forming",
-            default=0.05,
-            min=0.0,
-            soft_max=1.0
-            )
-    leafProbability = FloatProperty(
-            name="Leaf Probability",
-            description="Probability of a leaf forming",
-            default=0.35,
-            min=0.0,
-            soft_max=1.0
-            )
-    ivySize = FloatProperty(
-            name="Ivy Size",
-            description="The length of an ivy segment in Blender"
-                        " Units",
-            default=0.02,
-            min=0.0,
-            soft_max=1.0,
-            precision=3
-            )
-    ivyLeafSize = FloatProperty(
-            name="Ivy Leaf Size",
-            description="The size of the ivy leaves",
-            default=0.02,
-            min=0.0,
-            soft_max=0.5,
-            precision=3
-            )
-    ivyBranchSize = FloatProperty(
-            name="Ivy Branch Size",
-            description="The size of the ivy branches",
-            default=0.001,
-            min=0.0,
-            soft_max=0.1,
-            precision=4
-            )
-    maxFloatLength = FloatProperty(
-            name="Max Float Length",
-            description="The maximum distance that a branch "
-                        "can live while floating",
-            default=0.5,
-            min=0.0,
-            soft_max=1.0)
-    maxAdhesionDistance = FloatProperty(
-            name="Max Adhesion Length",
-            description="The maximum distance that a branch "
-                        "will feel the effects of adhesion",
-            default=1.0,
-            min=0.0,
-            soft_max=2.0,
-            precision=2
-            )
-    randomSeed = IntProperty(
-            name="Random Seed",
-            description="The seed governing random generation",
-            default=0,
-            min=0
-            )
-    maxTime = FloatProperty(
-            name="Maximum Time",
-            description="The maximum time to run the generation for "
-                        "in seconds generation (0.0 = Disabled)",
-            default=0.0,
-            min=0.0,
-            soft_max=10
-            )
-    growLeaves = BoolProperty(
-            name="Grow Leaves",
-            description="Grow leaves or not",
-            default=True
-            )
-    updateIvy = BoolProperty(
-            name="Update Ivy",
-            default=False
-            )
+    updateIvy: BoolProperty(
+        name="Update Ivy",
+        description="Update the Ivy location based on the cursor and Panel settings",
+        default=False
+    )
+    defaultIvy: BoolProperty(
+        name="Default Ivy",
+        options={"HIDDEN", "SKIP_SAVE"},
+        default=False
+    )
 
     @classmethod
     def poll(self, context):
@@ -598,14 +467,36 @@ class IvyGen(bpy.types.Operator):
         return self.execute(context)
 
     def execute(self, context):
+        # scene = context.scene
+        ivyProps = context.window_manager.ivy_gen_props
+
         if not self.updateIvy:
             return {'PASS_THROUGH'}
 
+        # assign the variables, check if it is default
+        # Note: update the values if window_manager props defaults are changed
+        randomSeed = ivyProps.randomSeed if not self.defaultIvy else 0
+        maxTime = ivyProps.maxTime if not self.defaultIvy else 0
+        maxIvyLength = ivyProps.maxIvyLength if not self.defaultIvy else 1.0
+        ivySize = ivyProps.ivySize if not self.defaultIvy else 0.02
+        maxFloatLength = ivyProps.maxFloatLength if not self.defaultIvy else 0.5
+        maxAdhesionDistance = ivyProps.maxAdhesionDistance if not self.defaultIvy else 1.0
+        primaryWeight = ivyProps.primaryWeight if not self.defaultIvy else 0.5
+        randomWeight = ivyProps.randomWeight if not self.defaultIvy else 0.2
+        gravityWeight = ivyProps.gravityWeight if not self.defaultIvy else 1.0
+        adhesionWeight = ivyProps.adhesionWeight if not self.defaultIvy else 0.1
+        branchingProbability = ivyProps.branchingProbability if not self.defaultIvy else 0.05
+        leafProbability = ivyProps.leafProbability if not self.defaultIvy else 0.35
+        ivyBranchSize = ivyProps.ivyBranchSize if not self.defaultIvy else 0.001
+        ivyLeafSize = ivyProps.ivyLeafSize if not self.defaultIvy else 0.02
+        growLeaves = ivyProps.growLeaves if not self.defaultIvy else True
+
         bpy.ops.object.mode_set(mode='EDIT', toggle=False)
         bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
 
         # Get the selected object
         ob = context.active_object
+        bvhtree = bvhtree_from_object(ob)
 
         # Check if the mesh has at least one polygon since some functions
         # are expecting them in the object's data (see T51753)
@@ -620,23 +511,33 @@ class IvyGen(bpy.types.Operator):
         # radius = computeBoundingSphere(ob)  # Not needed anymore
 
         # Get the seeding point
-        seedPoint = context.scene.cursor_location
+        seedPoint = context.scene.cursor.location
 
         # Fix the random seed
-        rand_seed(self.randomSeed)
+        rand_seed(randomSeed)
 
         # Make the new ivy
-        IVY = Ivy(**self.as_keywords(ignore=('randomSeed', 'growLeaves',
-                                  'maxIvyLength', 'maxTime', 'updateIvy')))
-
+        IVY = Ivy(
+            primaryWeight=primaryWeight,
+            randomWeight=randomWeight,
+            gravityWeight=gravityWeight,
+            adhesionWeight=adhesionWeight,
+            branchingProbability=branchingProbability,
+            leafProbability=leafProbability,
+            ivySize=ivySize,
+            ivyLeafSize=ivyLeafSize,
+            ivyBranchSize=ivyBranchSize,
+            maxFloatLength=maxFloatLength,
+            maxAdhesionDistance=maxAdhesionDistance
+            )
         # Generate first root and node
         IVY.seed(seedPoint)
 
         checkTime = False
-        maxLength = self.maxIvyLength  # * radius
+        maxLength = maxIvyLength  # * radius
 
         # If we need to check time set the flag
-        if self.maxTime != 0.0:
+        if maxTime != 0.0:
             checkTime = True
 
         t = time.time()
@@ -646,9 +547,9 @@ class IvyGen(bpy.types.Operator):
         # Grow until 200 roots is reached or backup counter exceeds limit
         while (any(checkAliveIter) and
                (IVY.maxLength < maxLength) and
-               (not checkTime or (time.time() - t < self.maxTime))):
+               (not checkTime or (time.time() - t < maxTime))):
             # Grow the ivy for this iteration
-            IVY.grow(ob)
+            IVY.grow(ob, bvhtree)
 
             # Print the proportion of ivy growth to console
             if (IVY.maxLength / maxLength * 100) > 10 * startPercent // 10:
@@ -662,86 +563,214 @@ class IvyGen(bpy.types.Operator):
             checkAliveIter = (r.alive for r in IVY.ivyRoots)
 
         # Create the curve and leaf geometry
-        createIvyGeometry(IVY, self.growLeaves)
+        createIvyGeometry(IVY, growLeaves)
         print("Geometry Generation Complete")
 
         print("Ivy generated in %0.2f s" % (time.time() - t))
 
         self.updateIvy = False
+        self.defaultIvy = False
 
         return {'FINISHED'}
 
     def draw(self, context):
         layout = self.layout
 
-        layout.prop(self, 'updateIvy', icon='CURVE_DATA')
+        layout.prop(self, "updateIvy", icon="FILE_REFRESH")
 
-        properties = layout.operator('curve.ivy_gen', text="Add New Ivy")
-        properties.randomSeed = self.randomSeed
-        properties.maxTime = self.maxTime
-        properties.maxIvyLength = self.maxIvyLength
-        properties.ivySize = self.ivySize
-        properties.maxFloatLength = self.maxFloatLength
-        properties.maxAdhesionDistance = self.maxAdhesionDistance
-        properties.primaryWeight = self.primaryWeight
-        properties.randomWeight = self.randomWeight
-        properties.gravityWeight = self.gravityWeight
-        properties.adhesionWeight = self.adhesionWeight
-        properties.branchingProbability = self.branchingProbability
-        properties.leafProbability = self.leafProbability
-        properties.ivyBranchSize = self.ivyBranchSize
-        properties.ivyLeafSize = self.ivyLeafSize
-        properties.updateIvy = True
 
-        prop_def = layout.operator('curve.ivy_gen', text="Add New Default Ivy")
+class CURVE_PT_IvyGenPanel(Panel):
+    bl_label = "Ivy Generator"
+    bl_idname = "CURVE_PT_IvyGenPanel"
+    bl_space_type = "VIEW_3D"
+    bl_region_type = "UI"
+    bl_category = "Create"
+    bl_context = "objectmode"
+    bl_options = {"DEFAULT_CLOSED"}
+
+    def draw(self, context):
+        layout = self.layout
+        wm = context.window_manager
+        col = layout.column(align=True)
+
+        prop_new = col.operator("curve.ivy_gen", text="Add New Ivy", icon="OUTLINER_OB_CURVE")
+        prop_new.defaultIvy = False
+        prop_new.updateIvy = True
+
+        prop_def = col.operator("curve.ivy_gen", text="Add New Default Ivy", icon="CURVE_DATA")
+        prop_def.defaultIvy = True
         prop_def.updateIvy = True
 
-        layout.prop(self, 'growLeaves')
+        col = layout.column(align=True)
+        col.label(text="Generation Settings:")
+        col.prop(wm.ivy_gen_props, "randomSeed")
+        col.prop(wm.ivy_gen_props, "maxTime")
 
-        box = layout.box()
-        box.label("Generation Settings:")
-        box.prop(self, 'randomSeed')
-        box.prop(self, 'maxTime')
+        col = layout.column(align=True)
+        col.label(text="Size Settings:")
+        col.prop(wm.ivy_gen_props, "maxIvyLength")
+        col.prop(wm.ivy_gen_props, "ivySize")
+        col.prop(wm.ivy_gen_props, "maxFloatLength")
+        col.prop(wm.ivy_gen_props, "maxAdhesionDistance")
 
-        box = layout.box()
-        box.label("Size Settings:")
-        box.prop(self, 'maxIvyLength')
-        box.prop(self, 'ivySize')
-        box.prop(self, 'maxFloatLength')
-        box.prop(self, 'maxAdhesionDistance')
+        col = layout.column(align=True)
+        col.label(text="Weight Settings:")
+        col.prop(wm.ivy_gen_props, "primaryWeight")
+        col.prop(wm.ivy_gen_props, "randomWeight")
+        col.prop(wm.ivy_gen_props, "gravityWeight")
+        col.prop(wm.ivy_gen_props, "adhesionWeight")
 
-        box = layout.box()
-        box.label("Weight Settings:")
-        box.prop(self, 'primaryWeight')
-        box.prop(self, 'randomWeight')
-        box.prop(self, 'gravityWeight')
-        box.prop(self, 'adhesionWeight')
+        col = layout.column(align=True)
+        col.label(text="Branch Settings:")
+        col.prop(wm.ivy_gen_props, "branchingProbability")
+        col.prop(wm.ivy_gen_props, "ivyBranchSize")
 
-        box = layout.box()
-        box.label("Branch Settings:")
-        box.prop(self, 'branchingProbability')
-        box.prop(self, 'ivyBranchSize')
+        col = layout.column(align=True)
+        col.prop(wm.ivy_gen_props, "growLeaves")
 
-        if self.growLeaves:
-            box = layout.box()
-            box.label("Leaf Settings:")
-            box.prop(self, 'ivyLeafSize')
-            box.prop(self, 'leafProbability')
+        if wm.ivy_gen_props.growLeaves:
+            col = layout.column(align=True)
+            col.label(text="Leaf Settings:")
+            col.prop(wm.ivy_gen_props, "ivyLeafSize")
+            col.prop(wm.ivy_gen_props, "leafProbability")
 
 
-def menu_func(self, context):
-    self.layout.operator(IvyGen.bl_idname, text="Add Ivy to Mesh",
-                         icon='OUTLINER_DATA_CURVE').updateIvy = True
+class IvyGenProperties(PropertyGroup):
+    maxIvyLength: FloatProperty(
+        name="Max Ivy Length",
+        description="Maximum ivy length in Blender Units",
+        default=1.0,
+        min=0.0,
+        soft_max=3.0,
+        subtype='DISTANCE',
+        unit='LENGTH'
+    )
+    primaryWeight: FloatProperty(
+        name="Primary Weight",
+        description="Weighting given to the current direction",
+        default=0.5,
+        min=0.0,
+        soft_max=1.0
+    )
+    randomWeight: FloatProperty(
+        name="Random Weight",
+        description="Weighting given to the random direction",
+        default=0.2,
+        min=0.0,
+        soft_max=1.0
+    )
+    gravityWeight: FloatProperty(
+        name="Gravity Weight",
+        description="Weighting given to the gravity direction",
+        default=1.0,
+        min=0.0,
+        soft_max=1.0
+    )
+    adhesionWeight: FloatProperty(
+        name="Adhesion Weight",
+        description="Weighting given to the adhesion direction",
+        default=0.1,
+        min=0.0,
+        soft_max=1.0
+    )
+    branchingProbability: FloatProperty(
+        name="Branching Probability",
+        description="Probability of a new branch forming",
+        default=0.05,
+        min=0.0,
+        soft_max=1.0
+    )
+    leafProbability: FloatProperty(
+        name="Leaf Probability",
+        description="Probability of a leaf forming",
+        default=0.35,
+        min=0.0,
+        soft_max=1.0
+    )
+    ivySize: FloatProperty(
+        name="Ivy Size",
+        description="The length of an ivy segment in Blender"
+                    " Units",
+        default=0.02,
+        min=0.0,
+        soft_max=1.0,
+        precision=3
+    )
+    ivyLeafSize: FloatProperty(
+        name="Ivy Leaf Size",
+        description="The size of the ivy leaves",
+        default=0.02,
+        min=0.0,
+        soft_max=0.5,
+        precision=3
+    )
+    ivyBranchSize: FloatProperty(
+        name="Ivy Branch Size",
+        description="The size of the ivy branches",
+        default=0.001,
+        min=0.0,
+        soft_max=0.1,
+        precision=4
+    )
+    maxFloatLength: FloatProperty(
+        name="Max Float Length",
+        description="The maximum distance that a branch "
+                    "can live while floating",
+        default=0.5,
+        min=0.0,
+        soft_max=1.0
+    )
+    maxAdhesionDistance: FloatProperty(
+        name="Max Adhesion Length",
+        description="The maximum distance that a branch "
+                    "will feel the effects of adhesion",
+        default=1.0,
+        min=0.0,
+        soft_max=2.0,
+        precision=2
+    )
+    randomSeed: IntProperty(
+        name="Random Seed",
+        description="The seed governing random generation",
+        default=0,
+        min=0
+    )
+    maxTime: FloatProperty(
+        name="Maximum Time",
+        description="The maximum time to run the generation for "
+                    "in seconds generation (0.0 = Disabled)",
+        default=0.0,
+        min=0.0,
+        soft_max=10
+    )
+    growLeaves: BoolProperty(
+        name="Grow Leaves",
+        description="Grow leaves or not",
+        default=True
+    )
+
+
+classes = (
+    IvyGen,
+    IvyGenProperties,
+    CURVE_PT_IvyGenPanel
+)
 
 
 def register():
-    bpy.utils.register_module(__name__)
-    bpy.types.INFO_MT_curve_add.append(menu_func)
+    for cls in classes:
+        bpy.utils.register_class(cls)
+
+    bpy.types.WindowManager.ivy_gen_props = PointerProperty(
+        type=IvyGenProperties
+    )
 
 
 def unregister():
-    bpy.types.INFO_MT_curve_add.remove(menu_func)
-    bpy.utils.unregister_module(__name__)
+    del bpy.types.WindowManager.ivy_gen_props
+
+    for cls in reversed(classes):
+        bpy.utils.unregister_class(cls)
 
 
 if __name__ == "__main__":
