@@ -1,22 +1,4 @@
-# ##### BEGIN GPL LICENSE BLOCK #####
-#
-#  This program is free software; you can redistribute it and/or
-#  modify it under the terms of the GNU General Public License
-#  as published by the Free Software Foundation; either version 2
-#  of the License, or (at your option) any later version.
-#
-#  This program is distributed in the hope that it will be useful,
-#  but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#  GNU General Public License for more details.
-#
-#  You should have received a copy of the GNU General Public License
-#  along with this program; if not, write to the Free Software Foundation,
-#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
-#
-# ##### END GPL LICENSE BLOCK #####
-
-# <pep8 compliant>
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 # Script copyright (C) Campbell Barton, Bastien Montagne
 
@@ -24,13 +6,13 @@
 import math
 import time
 
-from collections import namedtuple, OrderedDict
+from collections import namedtuple
 from collections.abc import Iterable
 from itertools import zip_longest, chain
 
 import bpy
 import bpy_extras
-from bpy.types import Object, Bone, PoseBone, DupliObject
+from bpy.types import Object, Bone, PoseBone, DepsgraphObjectInstance
 from mathutils import Vector, Matrix
 
 from . import encode_bin, data_types
@@ -51,6 +33,7 @@ FBX_GEOMETRY_NORMAL_VERSION = 101
 FBX_GEOMETRY_BINORMAL_VERSION = 101
 FBX_GEOMETRY_TANGENT_VERSION = 101
 FBX_GEOMETRY_SMOOTHING_VERSION = 102
+FBX_GEOMETRY_CREASE_VERSION = 101
 FBX_GEOMETRY_VCOLOR_VERSION = 101
 FBX_GEOMETRY_UV_VERSION = 101
 FBX_GEOMETRY_MATERIAL_VERSION = 101
@@ -71,7 +54,7 @@ FBX_ANIM_PROPSGROUP_NAME = "d"
 FBX_KTIME = 46186158000  # This is the number of "ktimes" in one second (yep, precision over the nanosecond...)
 
 
-MAT_CONVERT_LAMP = Matrix.Rotation(math.pi / 2.0, 4, 'X')  # Blender is -Z, FBX is -Y.
+MAT_CONVERT_LIGHT = Matrix.Rotation(math.pi / 2.0, 4, 'X')  # Blender is -Z, FBX is -Y.
 MAT_CONVERT_CAMERA = Matrix.Rotation(math.pi / 2.0, 4, 'Y')  # Blender is -Z, FBX is +X.
 # XXX I can't get this working :(
 # MAT_CONVERT_BONE = Matrix.Rotation(math.pi / 2.0, 4, 'Z')  # Blender is +Y, FBX is -X.
@@ -94,6 +77,7 @@ FBX_LIGHT_DECAY_TYPES = {
     'CONSTANT': 0,                   # None.
     'INVERSE_LINEAR': 1,             # Linear.
     'INVERSE_SQUARE': 2,             # Quadratic.
+    'INVERSE_COEFFICIENTS': 2,       # Quadratic...
     'CUSTOM_CURVE': 2,               # Quadratic.
     'LINEAR_QUADRATIC_WEIGHTED': 2,  # Quadratic.
 }
@@ -128,27 +112,35 @@ RIGHT_HAND_AXES = {
 }
 
 
+# NOTE: Not fully in enum value order, since when exporting the first entry matching the framerate value is used
+# (e.g. better have NTSC fullframe than NTSC drop frame for 29.97 framerate).
 FBX_FRAMERATES = (
+    #(-1.0, 0),  # Default framerate.
     (-1.0, 14),  # Custom framerate.
     (120.0, 1),
     (100.0, 2),
     (60.0, 3),
     (50.0, 4),
     (48.0, 5),
-    (30.0, 6),  # BW NTSC.
-    (30.0 / 1.001, 9),  # Color NTSC.
+    (30.0, 6),  # BW NTSC, full frame.
+    (30.0, 7),  # Drop frame.
+    (30.0 / 1.001, 9),  # Color NTSC, full frame.
+    (30.0 / 1.001, 8),  # Color NTSC, drop frame.
     (25.0, 10),
     (24.0, 11),
+    #(1.0, 12),  # 1000 milli/s (use for date time?).
     (24.0 / 1.001, 13),
     (96.0, 15),
     (72.0, 16),
     (60.0 / 1.001, 17),
+    (120.0 / 1.001, 18),
 )
 
 
 # ##### Misc utilities #####
 
-DO_PERFMON = True
+# Enable performance reports (measuring time used to perform various steps of importing or exporting).
+DO_PERFMON = False
 
 if DO_PERFMON:
     class PerfMon():
@@ -270,14 +262,14 @@ def similar_values_iter(v1, v2, e=1e-6):
 def vcos_transformed_gen(raw_cos, m=None):
     # Note: we could most likely get much better performances with numpy, but will leave this as TODO for now.
     gen = zip(*(iter(raw_cos),) * 3)
-    return gen if m is None else (m * Vector(v) for v in gen)
+    return gen if m is None else (m @ Vector(v) for v in gen)
 
 def nors_transformed_gen(raw_nors, m=None):
     # Great, now normals are also expected 4D!
     # XXX Back to 3D normals for now!
     # gen = zip(*(iter(raw_nors),) * 3 + (_infinite_gen(1.0),))
     gen = zip(*(iter(raw_nors),) * 3)
-    return gen if m is None else (m * Vector(v) for v in gen)
+    return gen if m is None else (m @ Vector(v) for v in gen)
 
 
 # ##### UIDs code. #####
@@ -323,7 +315,7 @@ def _key_to_uuid(uuids, key):
 
 def get_fbx_uuid_from_key(key):
     """
-    Return an UUID for given key, which is assumed hasable.
+    Return an UUID for given key, which is assumed to be hashable.
     """
     uuid = _keys_to_uuids.get(key, None)
     if uuid is None:
@@ -430,6 +422,10 @@ def get_blender_anim_curve_key(scene, ref_id, obj_key, fbx_prop_name, fbx_prop_i
                      fbx_prop_item_name, "AnimCurve"))
 
 
+def get_blender_nodetexture_key(ma, socket_names):
+    return "|".join((get_blenderID_key(ma), *socket_names))
+
+
 # ##### Element generators. #####
 
 # Note: elem may be None, in this case the element is not added to any parent.
@@ -588,10 +584,12 @@ def _elem_props_flags(animatable, animated, custom):
                 return b"A+U"
             return b"A+"
         if custom:
-            return b"AU"
+            # Seems that customprops always need those 'flags', see T69554. Go figure...
+            return b"A+U"
         return b"A"
     if custom:
-        return b"U"
+        # Seems that customprops always need those 'flags', see T69554. Go figure...
+        return b"A+U"
     return b""
 
 
@@ -613,12 +611,12 @@ def elem_props_template_init(templates, template_type):
     """
     Init a writing template of given type, for *one* element's properties.
     """
-    ret = OrderedDict()
+    ret = {}
     tmpl = templates.get(template_type)
     if tmpl is not None:
         written = tmpl.written[0]
         props = tmpl.properties
-        ret = OrderedDict((name, [val, ptype, anim, written]) for name, (val, ptype, anim) in props.items())
+        ret = {name: [val, ptype, anim, written] for name, (val, ptype, anim) in props.items()}
     return ret
 
 
@@ -649,7 +647,7 @@ def elem_props_template_finalize(template, elem):
     Finalize one element's template/props.
     Issue is, some templates might be "needed" by different types (e.g. NodeAttribute is for lights, cameras, etc.),
     but values for only *one* subtype can be written as template. So we have to be sure we write those for the other
-    subtypes in each and every elements, if they are not overriden by that element.
+    subtypes in each and every elements, if they are not overridden by that element.
     Yes, hairy, FBX that is to say. When they could easily support several subtypes per template... :(
     """
     for name, (value, ptype_name, animatable, written) in template.items():
@@ -670,14 +668,11 @@ def fbx_templates_generate(root, fbx_templates):
     # for Lights, Cameras, LibNodes, etc.).
     ref_templates = {(tmpl.type_name, tmpl.prop_type_name): tmpl for tmpl in fbx_templates.values()}
 
-    templates = OrderedDict()
+    templates = {}
     for type_name, prop_type_name, properties, nbr_users, _written in fbx_templates.values():
-        tmpl = templates.get(type_name)
-        if tmpl is None:
-            templates[type_name] = [OrderedDict(((prop_type_name, (properties, nbr_users)),)), nbr_users]
-        else:
-            tmpl[0][prop_type_name] = (properties, nbr_users)
-            tmpl[1] += nbr_users
+        tmpl = templates.setdefault(type_name, [{}, 0])
+        tmpl[0][prop_type_name] = (properties, nbr_users)
+        tmpl[1] += nbr_users
 
     for type_name, (subprops, nbr_users) in templates.items():
         template = elem_data_single_string(root, b"ObjectType", type_name)
@@ -728,6 +723,8 @@ class AnimationCurveNodeWrapper:
         'LCL_ROTATION': ("Lcl Rotation", "R", ("X", "Y", "Z")),
         'LCL_SCALING': ("Lcl Scaling", "S", ("X", "Y", "Z")),
         'SHAPE_KEY': ("DeformPercent", "DeformPercent", ("DeformPercent",)),
+        'CAMERA_FOCAL': ("FocalLength", "FocalLength", ("FocalLength",)),
+        'CAMERA_FOCUS_DISTANCE': ("FocusDistance", "FocusDistance", ("FocusDistance",)),
     }
 
     def __init__(self, elem_key, kind, force_keying, force_startend_keying, default_values=...):
@@ -842,7 +839,7 @@ class AnimationCurveNodeWrapper:
         for elem_key, fbx_group, fbx_gname, fbx_props in \
             zip(self.elem_keys, self.fbx_group, self.fbx_gname, self.fbx_props):
             group_key = get_blender_anim_curve_node_key(scene, ref_id, elem_key, fbx_group)
-            group = OrderedDict()
+            group = {}
             for c, def_val, fbx_item in zip(curves, self.default_values, fbx_props):
                 fbx_item = FBX_ANIM_PROPSGROUP_NAME + "|" + fbx_item
                 curve_key = get_blender_anim_curve_key(scene, ref_id, elem_key, fbx_group, fbx_item)
@@ -854,7 +851,7 @@ class AnimationCurveNodeWrapper:
 
 # ##### FBX objects generators. #####
 
-# FBX Model-like data (i.e. Blender objects, dupliobjects and bones) are wrapped in ObjectWrapper.
+# FBX Model-like data (i.e. Blender objects, depsgraph instances and bones) are wrapped in ObjectWrapper.
 # This allows us to have a (nearly) same code FBX-wise for all those types.
 # The wrapper tries to stay as small as possible, by mostly using callbacks (property(get...))
 # to actual Blender data it contains.
@@ -868,9 +865,13 @@ class MetaObjectWrapper(type):
         dup_mat = None
         if isinstance(bdata, Object):
             key = get_blenderID_key(bdata)
-        elif isinstance(bdata, DupliObject):
-            key = "|".join((get_blenderID_key((bdata.id_data, bdata.object)), cls._get_dup_num_id(bdata)))
-            dup_mat = bdata.matrix.copy()
+        elif isinstance(bdata, DepsgraphObjectInstance):
+            if bdata.is_instance:
+                key = "|".join((get_blenderID_key((bdata.parent.original, bdata.instance_object.original)),
+                                cls._get_dup_num_id(bdata)))
+                dup_mat = bdata.matrix_world.copy()
+            else:
+                key = get_blenderID_key(bdata.object.original)
         else:  # isinstance(bdata, (Bone, PoseBone)):
             if isinstance(bdata, PoseBone):
                 bdata = armature.data.bones[bdata.name]
@@ -881,9 +882,9 @@ class MetaObjectWrapper(type):
             cache = cls._cache = {}
         instance = cache.get(key)
         if instance is not None:
-            # Duplis hack: since duplis are not persistent in Blender (we have to re-create them to get updated
+            # Duplis hack: since dupli instances are not persistent in Blender (we have to re-create them to get updated
             # info like matrix...), we *always* need to reset that matrix when calling ObjectWrapper() (all
-            # other data is supposed valid during whole cache live, so we can skip resetting it).
+            # other data is supposed valid during whole cache live span, so we can skip resetting it).
             instance._dupli_matrix = dup_mat
             return instance
 
@@ -900,7 +901,7 @@ class ObjectWrapper(metaclass=MetaObjectWrapper):
     This class provides a same common interface for all (FBX-wise) object-like elements:
     * Blender Object
     * Blender Bone and PoseBone
-    * Blender DupliObject
+    * Blender DepsgraphObjectInstance (for dulis).
     Note since a same Blender object might be 'mapped' to several FBX models (esp. with duplis),
     we need to use a key to identify each.
     """
@@ -916,24 +917,42 @@ class ObjectWrapper(metaclass=MetaObjectWrapper):
 
     @staticmethod
     def _get_dup_num_id(bdata):
-        return ".".join(str(i) for i in bdata.persistent_id if i != 2147483647)
+        INVALID_IDS = {2147483647, 0}
+        pids = tuple(bdata.persistent_id)
+        idx_valid = 0
+        prev_i = ...
+        for idx, i in enumerate(pids[::-1]):
+            if i not in INVALID_IDS or (idx == len(pids) and i == 0 and prev_i != 0):
+                idx_valid = len(pids) - idx
+                break
+            prev_i = i
+        return ".".join(str(i) for i in pids[:idx_valid])
 
     def __init__(self, bdata, armature=None):
         """
-        bdata might be an Object, DupliObject, Bone or PoseBone.
+        bdata might be an Object (deprecated), DepsgraphObjectInstance, Bone or PoseBone.
         If Bone or PoseBone, armature Object must be provided.
         """
-        if isinstance(bdata, Object):
+        # Note: DepsgraphObjectInstance are purely runtime data, they become invalid as soon as we step to the next item!
+        #       Hence we have to immediately copy *all* needed data...
+        if isinstance(bdata, Object):  # DEPRECATED
             self._tag = 'OB'
             self.name = get_blenderID_name(bdata)
             self.bdata = bdata
             self._ref = None
-        elif isinstance(bdata, DupliObject):
-            self._tag = 'DP'
-            self.name = "|".join((get_blenderID_name((bdata.id_data, bdata.object)),
-                                  "Dupli", self._get_dup_num_id(bdata)))
-            self.bdata = bdata.object
-            self._ref = bdata.id_data
+        elif isinstance(bdata, DepsgraphObjectInstance):
+            if bdata.is_instance:
+                # Note that dupli instance matrix is set by meta-class initialization.
+                self._tag = 'DP'
+                self.name = "|".join((get_blenderID_name((bdata.parent.original, bdata.instance_object.original)),
+                                      "Dupli", self._get_dup_num_id(bdata)))
+                self.bdata = bdata.instance_object.original
+                self._ref = bdata.parent.original
+            else:
+                self._tag = 'OB'
+                self.name = get_blenderID_name(bdata)
+                self.bdata = bdata.object.original
+                self._ref = None
         else:  # isinstance(bdata, (Bone, PoseBone)):
             if isinstance(bdata, PoseBone):
                 bdata = armature.data.bones[bdata.name]
@@ -949,13 +968,17 @@ class ObjectWrapper(metaclass=MetaObjectWrapper):
     def __hash__(self):
         return hash(self.key)
 
+    def __repr__(self):
+        return self.key
+
     # #### Common to all _tag values.
     def get_fbx_uuid(self):
         return get_fbx_uuid_from_key(self.key)
     fbx_uuid = property(get_fbx_uuid)
 
+    # XXX Not sure how much that’s useful now... :/
     def get_hide(self):
-        return self.bdata.hide
+        return self.bdata.hide_viewport if self._tag in {'OB', 'DP'} else self.bdata.hide
     hide = property(get_hide)
 
     def get_parent(self):
@@ -972,21 +995,27 @@ class ObjectWrapper(metaclass=MetaObjectWrapper):
                 # Mere object parenting.
                 return ObjectWrapper(self.bdata.parent)
         elif self._tag == 'DP':
-            return ObjectWrapper(self.bdata.parent or self._ref)
+            return ObjectWrapper(self._ref)
         else:  # self._tag == 'BO'
             return ObjectWrapper(self.bdata.parent, self._ref) or ObjectWrapper(self._ref)
     parent = property(get_parent)
 
+    def get_bdata_pose_bone(self):
+        if self._tag == 'BO':
+            return self._ref.pose.bones[self.bdata.name]
+        return None
+    bdata_pose_bone = property(get_bdata_pose_bone)
+
     def get_matrix_local(self):
         if self._tag == 'OB':
             return self.bdata.matrix_local.copy()
         elif self._tag == 'DP':
-            return self._ref.matrix_world.inverted_safe() * self._dupli_matrix
+            return self._ref.matrix_world.inverted_safe() @ self._dupli_matrix
         else:  # 'BO', current pose
             # PoseBone.matrix is in armature space, bring in back in real local one!
             par = self.bdata.parent
             par_mat_inv = self._ref.pose.bones[par.name].matrix.inverted_safe() if par else Matrix()
-            return par_mat_inv * self._ref.pose.bones[self.bdata.name].matrix
+            return par_mat_inv @ self._ref.pose.bones[self.bdata.name].matrix
     matrix_local = property(get_matrix_local)
 
     def get_matrix_global(self):
@@ -995,7 +1024,7 @@ class ObjectWrapper(metaclass=MetaObjectWrapper):
         elif self._tag == 'DP':
             return self._dupli_matrix
         else:  # 'BO', current pose
-            return self._ref.matrix_world * self._ref.pose.bones[self.bdata.name].matrix
+            return self._ref.matrix_world @ self._ref.pose.bones[self.bdata.name].matrix
     matrix_global = property(get_matrix_global)
 
     def get_matrix_rest_local(self):
@@ -1003,14 +1032,14 @@ class ObjectWrapper(metaclass=MetaObjectWrapper):
             # Bone.matrix_local is in armature space, bring in back in real local one!
             par = self.bdata.parent
             par_mat_inv = par.matrix_local.inverted_safe() if par else Matrix()
-            return par_mat_inv * self.bdata.matrix_local
+            return par_mat_inv @ self.bdata.matrix_local
         else:
             return self.matrix_local.copy()
     matrix_rest_local = property(get_matrix_rest_local)
 
     def get_matrix_rest_global(self):
         if self._tag == 'BO':
-            return self._ref.matrix_world * self.bdata.matrix_local
+            return self._ref.matrix_world @ self.bdata.matrix_local
         else:
             return self.matrix_global.copy()
     matrix_rest_global = property(get_matrix_rest_global)
@@ -1063,41 +1092,41 @@ class ObjectWrapper(metaclass=MetaObjectWrapper):
         if self._tag == 'BO':
             # If we have a bone parent we need to undo the parent correction.
             if not is_global and scene_data.settings.bone_correction_matrix_inv and parent and parent.is_bone:
-                matrix = scene_data.settings.bone_correction_matrix_inv * matrix
+                matrix = scene_data.settings.bone_correction_matrix_inv @ matrix
             # Apply the bone correction.
             if scene_data.settings.bone_correction_matrix:
-                matrix = matrix * scene_data.settings.bone_correction_matrix
-        elif self.bdata.type == 'LAMP':
-            matrix = matrix * MAT_CONVERT_LAMP
+                matrix = matrix @ scene_data.settings.bone_correction_matrix
+        elif self.bdata.type == 'LIGHT':
+            matrix = matrix @ MAT_CONVERT_LIGHT
         elif self.bdata.type == 'CAMERA':
-            matrix = matrix * MAT_CONVERT_CAMERA
+            matrix = matrix @ MAT_CONVERT_CAMERA
 
         if self._tag in {'DP', 'OB'} and parent:
             if parent._tag == 'BO':
                 # In bone parent case, we get transformation in **bone tip** space (sigh).
                 # Have to bring it back into bone root, which is FBX expected value.
-                matrix = Matrix.Translation((0, (parent.bdata.tail - parent.bdata.head).length, 0)) * matrix
+                matrix = Matrix.Translation((0, (parent.bdata.tail - parent.bdata.head).length, 0)) @ matrix
 
         # Our matrix is in local space, time to bring it in its final desired space.
         if parent:
             if is_global:
                 # Move matrix to global Blender space.
-                matrix = (parent.matrix_rest_global if rest else parent.matrix_global) * matrix
+                matrix = (parent.matrix_rest_global if rest else parent.matrix_global) @ matrix
             elif parent.use_bake_space_transform(scene_data):
                 # Blender's and FBX's local space of parent may differ if we use bake_space_transform...
                 # Apply parent's *Blender* local space...
-                matrix = (parent.matrix_rest_local if rest else parent.matrix_local) * matrix
+                matrix = (parent.matrix_rest_local if rest else parent.matrix_local) @ matrix
                 # ...and move it back into parent's *FBX* local space.
                 par_mat = parent.fbx_object_matrix(scene_data, rest=rest, local_space=True)
-                matrix = par_mat.inverted_safe() * matrix
+                matrix = par_mat.inverted_safe() @ matrix
 
         if self.use_bake_space_transform(scene_data):
             # If we bake the transforms we need to post-multiply inverse global transform.
             # This means that the global transform will not apply to children of this transform.
-            matrix = matrix * scene_data.settings.global_matrix_inv
+            matrix = matrix @ scene_data.settings.global_matrix_inv
         if is_global:
             # In any case, pre-multiply the global matrix to get it in FBX global space!
-            matrix = scene_data.settings.global_matrix * matrix
+            matrix = scene_data.settings.global_matrix @ matrix
 
         return matrix
 
@@ -1158,23 +1187,15 @@ class ObjectWrapper(metaclass=MetaObjectWrapper):
         if self.parent == arm_obj and self.bdata.parent_type == 'ARMATURE':
             return True
         for mod in self.bdata.modifiers:
-            if mod.type == 'ARMATURE' and mod.object in {arm_obj.bdata, arm_obj.bdata.proxy}:
+            if mod.type == 'ARMATURE' and mod.object == arm_obj.bdata:
                 return True
 
     # #### Duplis...
-    def dupli_list_create(self, scene, settings='PREVIEW'):
-        if self._tag == 'OB' and self.bdata.is_duplicator:
-            self.bdata.dupli_list_create(scene, settings)
-
-    def dupli_list_clear(self):
-        if self._tag == 'OB'and self.bdata.is_duplicator:
-            self.bdata.dupli_list_clear()
-
-    def get_dupli_list(self):
-        if self._tag == 'OB'and self.bdata.is_duplicator:
-            return (ObjectWrapper(dup) for dup in self.bdata.dupli_list)
+    def dupli_list_gen(self, depsgraph):
+        if self._tag == 'OB' and self.bdata.is_instancer:
+            return (ObjectWrapper(dup) for dup in depsgraph.object_instances
+                                       if dup.parent and ObjectWrapper(dup.parent.original) == self)
         return ()
-    dupli_list = property(get_dupli_list)
 
 
 def fbx_name_class(name, cls):
@@ -1194,7 +1215,7 @@ FBXExportSettings = namedtuple("FBXExportSettings", (
     "report", "to_axes", "global_matrix", "global_scale", "apply_unit_scale", "unit_scale",
     "bake_space_transform", "global_matrix_inv", "global_matrix_inv_transposed",
     "context_objects", "object_types", "use_mesh_modifiers", "use_mesh_modifiers_render",
-    "mesh_smooth_type", "use_mesh_edges", "use_tspace",
+    "mesh_smooth_type", "use_subsurf", "use_mesh_edges", "use_tspace", "use_triangles",
     "armature_nodetype", "use_armature_deform_only", "add_leaf_bones",
     "bone_correction_matrix", "bone_correction_matrix_inv",
     "bake_anim", "bake_anim_use_all_bones", "bake_anim_use_nla_strips", "bake_anim_use_all_actions",
@@ -1211,8 +1232,8 @@ FBXExportSettings = namedtuple("FBXExportSettings", (
 #     * animations.
 FBXExportData = namedtuple("FBXExportData", (
     "templates", "templates_users", "connections",
-    "settings", "scene", "objects", "animations", "animated", "frame_start", "frame_end",
-    "data_empties", "data_lamps", "data_cameras", "data_meshes", "mesh_mat_indices",
+    "settings", "scene", "depsgraph", "objects", "animations", "animated", "frame_start", "frame_end",
+    "data_empties", "data_lights", "data_cameras", "data_meshes", "mesh_material_indices",
     "data_bones", "data_leaf_bones", "data_deformers_skin", "data_deformers_shape",
     "data_world", "data_materials", "data_textures", "data_videos",
 ))
@@ -1221,11 +1242,12 @@ FBXExportData = namedtuple("FBXExportData", (
 FBXImportSettings = namedtuple("FBXImportSettings", (
     "report", "to_axes", "global_matrix", "global_scale",
     "bake_space_transform", "global_matrix_inv", "global_matrix_inv_transposed",
-    "use_custom_normals", "use_cycles", "use_image_search",
+    "use_custom_normals", "use_image_search",
     "use_alpha_decals", "decal_offset",
     "use_anim", "anim_offset",
+    "use_subsurf",
     "use_custom_props", "use_custom_props_enum_as_string",
-    "cycles_material_wrap_map", "image_cache",
+    "nodal_material_wrap_map", "image_cache",
     "ignore_leaf_bones", "force_connect_children", "automatic_bone_orientation", "bone_correction_matrix",
     "use_prepost_rot",
 ))
