@@ -1,24 +1,10 @@
 #!/usr/local/bin/python3.10
-#
+# SPDX-License-Identifier: GPL-2.0-or-later
+# Copyright 2010  Maximilian Hoegner <hp.maxi@hoegners.de>.
+
 # geo.py is a python module with no dependencies on extra packages,
 # providing some convenience functions for working with geographic
 # coordinates
-#
-# Copyright (C) 2010  Maximilian Hoegner <hp.maxi@hoegners.de>
-#
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#
 
 ### Part one - Functions for dealing with points on a sphere ###
 
@@ -51,7 +37,7 @@ class Parser:
         # build pattern with subgroups
         sub_dict = {}
         subpattern_names = []
-        for s in re.finditer("%\(.*?\)s", self.raw_patterns[pattern_name]):
+        for s in re.finditer(r"%\(.*?\)s", self.raw_patterns[pattern_name]):
             subpattern_name = s.group()[2:-2]
             if not self.virtual[subpattern_name]:
                 sub_dict[subpattern_name] = "(" + self.patterns[
@@ -108,7 +94,7 @@ position_parser.add("minutes", r"%(number)s\s*%(minutes_symbol)s")
 position_parser.add("seconds", r"%(number)s\s*%(seconds_symbol)s")
 position_parser.add(
     "degree_coordinates",
-    "(?:%(sign)s\s*)?%(degrees)s(?:[+\s]*%(minutes)s)?(?:[+\s]*%(seconds)s)?|(?:%(sign)s\s*)%(minutes)s(?:[+\s]*%(seconds)s)?|(?:%(sign)s\s*)%(seconds)s"
+    r"(?:%(sign)s\s*)?%(degrees)s(?:[+\s]*%(minutes)s)?(?:[+\s]*%(seconds)s)?|(?:%(sign)s\s*)%(minutes)s(?:[+\s]*%(seconds)s)?|(?:%(sign)s\s*)%(seconds)s"
 )
 
 position_parser.add(
@@ -119,13 +105,13 @@ position_parser.add(
     r"%(nmea_style)s|%(plain_degrees)s|%(degree_coordinates)s")
 
 position_parser.add(
-    "position", """\
-\s*%(direction_ns)s\s*%(coordinates_ns)s[,;\s]*%(direction_ew)s\s*%(coordinates_ew)s\s*|\
-\s*%(direction_ew)s\s*%(coordinates_ew)s[,;\s]*%(direction_ns)s\s*%(coordinates_ns)s\s*|\
-\s*%(coordinates_ns)s\s*%(direction_ns)s[,;\s]*%(coordinates_ew)s\s*%(direction_ew)s\s*|\
-\s*%(coordinates_ew)s\s*%(direction_ew)s[,;\s]*%(coordinates_ns)s\s*%(direction_ns)s\s*|\
-\s*%(coordinates_ns)s[,;\s]+%(coordinates_ew)s\s*\
-""")
+    "position", (
+        r"\s*%(direction_ns)s\s*%(coordinates_ns)s[,;\s]*%(direction_ew)s\s*%(coordinates_ew)s\s*|"
+        r"\s*%(direction_ew)s\s*%(coordinates_ew)s[,;\s]*%(direction_ns)s\s*%(coordinates_ns)s\s*|"
+        r"\s*%(coordinates_ns)s\s*%(direction_ns)s[,;\s]*%(coordinates_ew)s\s*%(direction_ew)s\s*|"
+        r"\s*%(coordinates_ew)s\s*%(direction_ew)s[,;\s]*%(coordinates_ns)s\s*%(direction_ns)s\s*|"
+        r"\s*%(coordinates_ns)s[,;\s]+%(coordinates_ew)s\s*"
+    ))
 
 
 def get_number(b):
