@@ -1,20 +1,4 @@
-# ##### BEGIN GPL LICENSE BLOCK #####
-#
-#  This program is free software; you can redistribute it and/or
-#  modify it under the terms of the GNU General Public License
-#  as published by the Free Software Foundation; either version 2
-#  of the License, or (at your option) any later version.
-#
-#  This program is distributed in the hope that it will be useful,
-#  but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#  GNU General Public License for more details.
-#
-#  You should have received a copy of the GNU General Public License
-#  along with this program; if not, write to the Free Software Foundation,
-#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
-#
-# ##### END GPL LICENSE BLOCK #####
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 #  Filename : shaders.py
 #  Authors  : Fredo Durand, Stephane Grabli, Francois Sillion, Emmanuel Turquin
@@ -22,10 +6,74 @@
 #  Purpose  : Stroke shaders to be used for creation of stylized strokes
 
 """
-Stroke shaders used for creation of stylized strokes.  Also intended
-to be a collection of examples for shader definition in Python
+This module contains stroke shaders used for creation of stylized
+strokes.  It is also intended to be a collection of examples for
+shader definition in Python.
+
+User-defined stroke shaders inherit the
+:class:`freestyle.types.StrokeShader` class.
 """
 
+__all__ = (
+    "BackboneStretcherShader",
+    "BezierCurveShader",
+    "BlenderTextureShader",
+    "CalligraphicShader",
+    "ColorNoiseShader",
+    "ConstantColorShader",
+    "ConstantThicknessShader",
+    "ConstrainedIncreasingThicknessShader",
+    "GuidingLinesShader",
+    "IncreasingColorShader",
+    "IncreasingThicknessShader",
+    "PolygonalizationShader",
+    "RoundCapShader",
+    "SamplingShader",
+    "SmoothingShader",
+    "SpatialNoiseShader",
+    "SquareCapShader",
+    "StrokeTextureStepShader",
+    "ThicknessNoiseShader",
+    "TipRemoverShader",
+    "py2DCurvatureColorShader",
+    "pyBackboneStretcherNoCuspShader",
+    "pyBackboneStretcherShader",
+    "pyBluePrintCirclesShader",
+    "pyBluePrintDirectedSquaresShader",
+    "pyBluePrintEllipsesShader",
+    "pyBluePrintSquaresShader",
+    "pyConstantColorShader",
+    "pyConstantThicknessShader",
+    "pyConstrainedIncreasingThicknessShader",
+    "pyDecreasingThicknessShader",
+    "pyDepthDiscontinuityThicknessShader",
+    "pyDiffusion2Shader",
+    "pyFXSVaryingThicknessWithDensityShader",
+    "pyGuidingLineShader",
+    "pyHLRShader",
+    "pyImportance2DThicknessShader",
+    "pyImportance3DThicknessShader",
+    "pyIncreasingColorShader",
+    "pyIncreasingThicknessShader",
+    "pyInterpolateColorShader",
+    "pyLengthDependingBackboneStretcherShader",
+    "pyMaterialColorShader",
+    "pyModulateAlphaShader",
+    "pyNonLinearVaryingThicknessShader",
+    "pyPerlinNoise1DShader",
+    "pyPerlinNoise2DShader",
+    "pyRandomColorShader",
+    "pySLERPThicknessShader",
+    "pySamplingShader",
+    "pySinusDisplacementShader",
+    "pyTVertexRemoverShader",
+    "pyTVertexThickenerShader",
+    "pyTimeColorShader",
+    "pyTipRemoverShader",
+    "pyZDependingThicknessShader",
+)
+
+
 # module members
 from _freestyle import (
     BackboneStretcherShader,
@@ -33,7 +81,6 @@ from _freestyle import (
     BlenderTextureShader,
     CalligraphicShader,
     ColorNoiseShader,
-    ColorVariationPatternShader,
     ConstantColorShader,
     ConstantThicknessShader,
     ConstrainedIncreasingThicknessShader,
@@ -44,15 +91,10 @@ from _freestyle import (
     SamplingShader,
     SmoothingShader,
     SpatialNoiseShader,
-    StrokeTextureShader,
     StrokeTextureStepShader,
-    TextureAssignerShader,
     ThicknessNoiseShader,
-    ThicknessVariationPatternShader,
     TipRemoverShader,
-    fstreamShader,
-    streamShader,
-    )
+)
 
 # constructs for shader definition in Python
 from freestyle.types import (
@@ -63,7 +105,7 @@ from freestyle.types import (
     StrokeShader,
     StrokeVertexIterator,
     StrokeVertex,
-    )
+)
 from freestyle.functions import (
     Curvature2DAngleF0D,
     DensityF0D,
@@ -72,17 +114,18 @@ from freestyle.functions import (
     Normal2DF0D,
     Orientation2DF1D,
     ZDiscontinuityF0D,
-    )
+)
 from freestyle.predicates import (
     pyVertexNatureUP0D,
     pyUEqualsUP0D,
-    )
+)
 
 from freestyle.utils import (
     bound,
-    bounding_box,
+    BoundingBox,
+    pairwise,
     phase_to_direction,
-    )
+)
 
 from freestyle.utils import ContextFunctions as CF
 
@@ -100,8 +143,9 @@ from random import randint
 class pyDepthDiscontinuityThicknessShader(StrokeShader):
     """
     Assigns a thickness to the stroke based on the stroke's distance
-    to the camera (Z-value)
+    to the camera (Z-value).
     """
+
     def __init__(self, min, max):
         StrokeShader.__init__(self)
         self.a = max - min
@@ -118,8 +162,9 @@ class pyDepthDiscontinuityThicknessShader(StrokeShader):
 
 class pyConstantThicknessShader(StrokeShader):
     """
-    Assigns a constant thickness along the stroke
+    Assigns a constant thickness along the stroke.
     """
+
     def __init__(self, thickness):
         StrokeShader.__init__(self)
         self._thickness = thickness / 2.0
@@ -131,8 +176,9 @@ class pyConstantThicknessShader(StrokeShader):
 
 class pyFXSVaryingThicknessWithDensityShader(StrokeShader):
     """
-    Assings thickness to a stroke based on the density of the diffuse map
+    Assigns thickness to a stroke based on the density of the diffuse map.
     """
+
     def __init__(self, wsize, threshold_min, threshold_max, thicknessMin, thicknessMax):
         StrokeShader.__init__(self)
         self._func = DensityF0D(wsize)
@@ -155,8 +201,9 @@ class pyFXSVaryingThicknessWithDensityShader(StrokeShader):
 
 class pyIncreasingThicknessShader(StrokeShader):
     """
-    Increasingly thickens the stroke
+    Increasingly thickens the stroke.
     """
+
     def __init__(self, thicknessMin, thicknessMax):
         StrokeShader.__init__(self)
         self._thicknessMin = thicknessMin
@@ -176,8 +223,9 @@ class pyIncreasingThicknessShader(StrokeShader):
 class pyConstrainedIncreasingThicknessShader(StrokeShader):
     """
     Increasingly thickens the stroke, constrained by a ratio of the
-    stroke's length
+    stroke's length.
     """
+
     def __init__(self, thicknessMin, thicknessMax, ratio):
         StrokeShader.__init__(self)
         self._thicknessMin = thicknessMin
@@ -203,8 +251,9 @@ class pyConstrainedIncreasingThicknessShader(StrokeShader):
 
 class pyDecreasingThicknessShader(StrokeShader):
     """
-    Inverse of pyIncreasingThicknessShader, decreasingly thickens the stroke
+    Inverse of pyIncreasingThicknessShader, decreasingly thickens the stroke.
     """
+
     def __init__(self, thicknessMin, thicknessMax):
         StrokeShader.__init__(self)
         self._thicknessMin = thicknessMin
@@ -224,8 +273,9 @@ class pyDecreasingThicknessShader(StrokeShader):
 
 class pyNonLinearVaryingThicknessShader(StrokeShader):
     """
-    Assigns thickness to a stroke based on an exponential function
+    Assigns thickness to a stroke based on an exponential function.
     """
+
     def __init__(self, thicknessExtremity, thicknessMiddle, exponent):
         self._thicknessMin = thicknessMiddle
         self._thicknessMax = thicknessExtremity
@@ -243,8 +293,9 @@ class pyNonLinearVaryingThicknessShader(StrokeShader):
 
 class pySLERPThicknessShader(StrokeShader):
     """
-    Assigns thickness to a stroke based on spherical linear interpolation
+    Assigns thickness to a stroke based on spherical linear interpolation.
     """
+
     def __init__(self, thicknessMin, thicknessMax, omega=1.2):
         StrokeShader.__init__(self)
         self._thicknessMin = thicknessMin
@@ -259,16 +310,17 @@ class pySLERPThicknessShader(StrokeShader):
         for i, svert in enumerate(stroke):
             c = i / n
             if i < (n * 0.5):
-                t = sin((1-c) * omega) / sinhyp * self._thicknessMin + sin(c * omega) / sinhyp * maxT
+                t = sin((1 - c) * omega) / sinhyp * self._thicknessMin + sin(c * omega) / sinhyp * maxT
             else:
-                t = sin((1-c) * omega) / sinhyp * maxT + sin(c * omega) / sinhyp * self._thicknessMin
+                t = sin((1 - c) * omega) / sinhyp * maxT + sin(c * omega) / sinhyp * self._thicknessMin
             svert.attribute.thickness = (t / 2.0, t / 2.0)
 
 
 class pyTVertexThickenerShader(StrokeShader):
     """
-    Thickens TVertices (visual intersections between two edges)
+    Thickens TVertices (visual intersections between two edges).
     """
+
     def __init__(self, a=1.5, n=3):
         StrokeShader.__init__(self)
         self._a = a
@@ -297,8 +349,9 @@ class pyImportance2DThicknessShader(StrokeShader):
     """
     Assigns thickness based on distance to a given point in 2D space.
     the thickness is inverted, so the vertices closest to the
-    specified point have the lowest thickness
+    specified point have the lowest thickness.
     """
+
     def __init__(self, x, y, w, kmin, kmax):
         StrokeShader.__init__(self)
         self._origin = Vector((x, y))
@@ -309,16 +362,17 @@ class pyImportance2DThicknessShader(StrokeShader):
         for svert in stroke:
             d = (svert.point_2d - self._origin).length
             k = (self._kmin if (d > self._w) else
-                (self._kmax * (self._w-d) + self._kmin * d) / self._w)
+                 (self._kmax * (self._w - d) + self._kmin * d) / self._w)
 
             (tr, tl) = svert.attribute.thickness
-            svert.attribute.thickness = (k*tr/2.0, k*tl/2.0)
+            svert.attribute.thickness = (k * tr / 2.0, k * tl / 2.0)
 
 
 class pyImportance3DThicknessShader(StrokeShader):
     """
-    Assigns thickness based on distance to a given point in 3D space
+    Assigns thickness based on distance to a given point in 3D space.
     """
+
     def __init__(self, x, y, z, w, kmin, kmax):
         StrokeShader.__init__(self)
         self._origin = Vector((x, y, z))
@@ -329,17 +383,18 @@ class pyImportance3DThicknessShader(StrokeShader):
         for svert in stroke:
             d = (svert.point_3d - self._origin).length
             k = (self._kmin if (d > self._w) else
-                (self._kmax * (self._w-d) + self._kmin * d) / self._w)
+                 (self._kmax * (self._w - d) + self._kmin * d) / self._w)
 
             (tr, tl) = svert.attribute.thickness
-            svert.attribute.thickness = (k*tr/2.0, k*tl/2.0)
+            svert.attribute.thickness = (k * tr / 2.0, k * tl / 2.0)
 
 
 class pyZDependingThicknessShader(StrokeShader):
     """
     Assigns thickness based on an object's local Z depth (point
-    closest to camera is 1, point furthest from camera is zero)
+    closest to camera is 1, point furthest from camera is zero).
     """
+
     def __init__(self, min, max):
         StrokeShader.__init__(self)
         self.__min = min
@@ -363,12 +418,14 @@ class pyZDependingThicknessShader(StrokeShader):
 
 class pyConstantColorShader(StrokeShader):
     """
-    Assigns a constant color to the stroke
+    Assigns a constant color to the stroke.
     """
-    def __init__(self,r,g,b, a = 1):
+
+    def __init__(self, r, g, b, a=1):
         StrokeShader.__init__(self)
         self._color = (r, g, b)
         self._a = a
+
     def shade(self, stroke):
         for svert in stroke:
             svert.attribute.color = self._color
@@ -377,12 +434,13 @@ class pyConstantColorShader(StrokeShader):
 
 class pyIncreasingColorShader(StrokeShader):
     """
-    Fades from one color to another along the stroke
+    Fades from one color to another along the stroke.
     """
-    def __init__(self,r1,g1,b1,a1, r2,g2,b2,a2):
+
+    def __init__(self, r1, g1, b1, a1, r2, g2, b2, a2):
         StrokeShader.__init__(self)
         # use 4d vector to simplify math
-        self._c1 = Vector((r1, g1 ,b1, a1))
+        self._c1 = Vector((r1, g1, b1, a1))
         self._c2 = Vector((r2, g2, b2, a2))
 
     def shade(self, stroke):
@@ -397,12 +455,13 @@ class pyIncreasingColorShader(StrokeShader):
 
 class pyInterpolateColorShader(StrokeShader):
     """
-    Fades from one color to another and back
+    Fades from one color to another and back.
     """
-    def __init__(self,r1,g1,b1,a1, r2,g2,b2,a2):
+
+    def __init__(self, r1, g1, b1, a1, r2, g2, b2, a2):
         StrokeShader.__init__(self)
         # use 4d vector to simplify math
-        self._c1 = Vector((r1, g1 ,b1, a1))
+        self._c1 = Vector((r1, g1, b1, a1))
         self._c2 = Vector((r2, g2, b2, a2))
 
     def shade(self, stroke):
@@ -418,10 +477,12 @@ class pyModulateAlphaShader(StrokeShader):
     """
     Limits the stroke's alpha between a min and max value.
     """
+
     def __init__(self, min=0, max=1):
         StrokeShader.__init__(self)
         self.__min = min
         self.__max = max
+
     def shade(self, stroke):
         for svert in stroke:
             alpha = svert.attribute.alpha
@@ -431,8 +492,9 @@ class pyModulateAlphaShader(StrokeShader):
 
 class pyMaterialColorShader(StrokeShader):
     """
-    Assigns the color of the underlying material to the stroke
+    Assigns the color of the underlying material to the stroke.
     """
+
     def __init__(self, threshold=50):
         StrokeShader.__init__(self)
         self._threshold = threshold
@@ -461,7 +523,7 @@ class pyMaterialColorShader(StrokeShader):
             u = 4.0 * X / (X + 15.0 * Y + 3.0 * Z)
             v = 9.0 * Y / (X + 15.0 * Y + 3.0 * Z)
 
-            L= 116. * pow((Y/Yn),(1./3.)) - 16
+            L = 116. * pow((Y / Yn), (1. / 3.)) - 16
             U = 13. * L * (u - un)
             V = 13. * L * (v - vn)
 
@@ -469,16 +531,16 @@ class pyMaterialColorShader(StrokeShader):
                 L /= 1.3
                 U += 10.
             else:
-                L = L + 2.5 * (100-L) * 0.2
+                L = L + 2.5 * (100 - L) * 0.2
                 U /= 3.0
                 V /= 3.0
 
             u = U / (13.0 * L) + un
             v = V / (13.0 * L) + vn
 
-            Y = Yn * pow(((L+16.)/116.), 3.)
-            X = -9. * Y * u / ((u - 4.)* v - u * v)
-            Z = (9. * Y - 15*v*Y - v*X) /( 3. * v)
+            Y = Yn * pow(((L + 16.) / 116.), 3.)
+            X = -9. * Y * u / ((u - 4.) * v - u * v)
+            Z = (9. * Y - 15 * v * Y - v * X) / (3. * v)
 
             r = 3.240479 * X - 1.53715 * Y - 0.498535 * Z
             g = -0.969256 * X + 1.875991 * Y + 0.041556 * Z
@@ -493,8 +555,9 @@ class pyMaterialColorShader(StrokeShader):
 
 class pyRandomColorShader(StrokeShader):
     """
-    Assigns a color to the stroke based on given seed
+    Assigns a color to the stroke based on given seed.
     """
+
     def __init__(self, s=1):
         StrokeShader.__init__(self)
         random.seed = s
@@ -509,9 +572,10 @@ class pyRandomColorShader(StrokeShader):
 
 class py2DCurvatureColorShader(StrokeShader):
     """
-    Assigns a color (greyscale) to the stroke based on the curvature.
-    A higher curvature will yield a brighter color
+    Assigns a color (grayscale) to the stroke based on the curvature.
+    A higher curvature will yield a brighter color.
     """
+
     def shade(self, stroke):
         func = Curvature2DAngleF0D()
         it = Interface0DIterator(stroke)
@@ -525,12 +589,14 @@ class py2DCurvatureColorShader(StrokeShader):
 
 class pyTimeColorShader(StrokeShader):
     """
-    Assigns a greyscale value that increases for every vertex.
-    The brightness will increase along the stroke
+    Assigns a grayscale value that increases for every vertex.
+    The brightness will increase along the stroke.
     """
+
     def __init__(self, step=0.01):
         StrokeShader.__init__(self)
         self._step = step
+
     def shade(self, stroke):
         for i, svert in enumerate(stroke):
             c = i * self._step
@@ -542,9 +608,10 @@ class pyTimeColorShader(StrokeShader):
 
 class pySamplingShader(StrokeShader):
     """
-    Resamples the stroke, which gives the stroke the ammount of
-    vertices specified
+    Resamples the stroke, which gives the stroke the amount of
+    vertices specified.
     """
+
     def __init__(self, sampling):
         StrokeShader.__init__(self)
         self._sampling = sampling
@@ -556,8 +623,9 @@ class pySamplingShader(StrokeShader):
 
 class pyBackboneStretcherShader(StrokeShader):
     """
-    Stretches the stroke's backbone by a given length (in pixels)
+    Stretches the stroke's backbone by a given length (in pixels).
     """
+
     def __init__(self, l):
         StrokeShader.__init__(self)
         self._l = l
@@ -567,7 +635,7 @@ class pyBackboneStretcherShader(StrokeShader):
         v0, vn = stroke[0], stroke[-1]
         p0, pn = v0.point, vn.point
         # get the direction
-        d1 = (p0 - stroke[ 1].point).normalized()
+        d1 = (p0 - stroke[1].point).normalized()
         dn = (pn - stroke[-2].point).normalized()
         v0.point += d1 * self._l
         vn.point += dn * self._l
@@ -580,15 +648,17 @@ class pyLengthDependingBackboneStretcherShader(StrokeShader):
     NOTE: you'll probably want an l somewhere between (0.5 - 0). A value that
     is too high may yield unexpected results.
     """
+
     def __init__(self, l):
         StrokeShader.__init__(self)
         self._l = l
+
     def shade(self, stroke):
         # get start and end points
         v0, vn = stroke[0], stroke[-1]
         p0, pn = v0.point, vn.point
         # get the direction
-        d1 = (p0 - stroke[ 1].point).normalized()
+        d1 = (p0 - stroke[1].point).normalized()
         dn = (pn - stroke[-2].point).normalized()
         v0.point += d1 * self._l * stroke.length_2d
         vn.point += dn * self._l * stroke.length_2d
@@ -617,8 +687,9 @@ class pyGuidingLineShader(StrokeShader):
 
 class pyBackboneStretcherNoCuspShader(StrokeShader):
     """
-    Stretches the stroke's backbone, excluding cusp vertices (end junctions)
+    Stretches the stroke's backbone, excluding cusp vertices (end junctions).
     """
+
     def __init__(self, l):
         StrokeShader.__init__(self)
         self._l = l
@@ -646,6 +717,7 @@ class pyDiffusion2Shader(StrokeShader):
     point. The offset is scaled by the 2D curvature (i.e. how quickly
     the stroke curve is) at the point.
     """
+
     def __init__(self, lambda1, nbIter):
         StrokeShader.__init__(self)
         self._lambda = lambda1
@@ -654,7 +726,7 @@ class pyDiffusion2Shader(StrokeShader):
         self._curvatureInfo = Curvature2DAngleF0D()
 
     def shade(self, stroke):
-        for i in range (1, self._nbIter):
+        for _i in range(1, self._nbIter):
             it = Interface0DIterator(stroke)
             for svert in it:
                 svert.point += self._normalInfo(it) * self._lambda * self._curvatureInfo(it)
@@ -663,19 +735,18 @@ class pyDiffusion2Shader(StrokeShader):
 
 class pyTipRemoverShader(StrokeShader):
     """
-    Removes the tips of the stroke
+    Removes the tips of the stroke.
     """
+
     def __init__(self, l):
         StrokeShader.__init__(self)
         self._l = l
 
     @staticmethod
     def check_vertex(v, length):
-        """
-        Returns True if the given strokevertex is less than self._l away
-        from the stroke's tip and therefore should be removed.
-        """
-        return (v.curvilinear_abscissa < length or v.stroke_length-v.curvilinear_abscissa < length)
+        # Returns True if the given strokevertex is less than self._l away
+        # from the stroke's tip and therefore should be removed.
+        return (v.curvilinear_abscissa < length or v.stroke_length - v.curvilinear_abscissa < length)
 
     def shade(self, stroke):
         n = len(stroke)
@@ -704,8 +775,9 @@ class pyTipRemoverShader(StrokeShader):
 
 class pyTVertexRemoverShader(StrokeShader):
     """
-    Removes t-vertices from the stroke
+    Removes t-vertices from the stroke.
     """
+
     def shade(self, stroke):
         if len(stroke) < 4:
             return
@@ -720,9 +792,10 @@ class pyTVertexRemoverShader(StrokeShader):
 
 class pyHLRShader(StrokeShader):
     """
-    Controlls visibility based upon the quantative invisibility (QI)
-    based on hidden line removal (HLR)
+    Controls visibility based upon the quantitative invisibility (QI)
+    based on hidden line removal (HLR).
     """
+
     def shade(self, stroke):
         if len(stroke) < 4:
             return
@@ -736,8 +809,9 @@ class pyHLRShader(StrokeShader):
 
 class pySinusDisplacementShader(StrokeShader):
     """
-    Displaces the stroke in the shape of a sine wave
+    Displaces the stroke in the shape of a sine wave.
     """
+
     def __init__(self, f, a):
         StrokeShader.__init__(self)
         self._f = f
@@ -758,8 +832,9 @@ class pyPerlinNoise1DShader(StrokeShader):
     """
     Displaces the stroke using the curvilinear abscissa.  This means
     that lines with the same length and sampling interval will be
-    identically distorded
+    identically distorded.
     """
+
     def __init__(self, freq=10, amp=10, oct=4, seed=-1):
         StrokeShader.__init__(self)
         self.__noise = Noise(seed)
@@ -778,11 +853,12 @@ class pyPerlinNoise1DShader(StrokeShader):
 class pyPerlinNoise2DShader(StrokeShader):
     """
     Displaces the stroke using the strokes coordinates.  This means
-    that in a scene no strokes will be distorded identically
+    that in a scene no strokes will be distorded identically.
 
-    More information on the noise shaders can be found at
+    More information on the noise shaders can be found at:
     freestyleintegration.wordpress.com/2011/09/25/development-updates-on-september-25/
     """
+
     def __init__(self, freq=10, amp=10, oct=4, seed=-1):
         StrokeShader.__init__(self)
         self.__noise = Noise(seed)
@@ -799,8 +875,9 @@ class pyPerlinNoise2DShader(StrokeShader):
 
 class pyBluePrintCirclesShader(StrokeShader):
     """
-    Draws the silhouette of the object as a circle
+    Draws the silhouette of the object as a circle.
     """
+
     def __init__(self, turns=1, random_radius=3, random_center=5):
         StrokeShader.__init__(self)
         self.__turns = turns
@@ -809,7 +886,7 @@ class pyBluePrintCirclesShader(StrokeShader):
 
     def shade(self, stroke):
         # get minimum and maximum coordinates
-        p_min, p_max = bounding_box(stroke)
+        p_min, p_max = BoundingBox.from_sequence(svert.point for svert in stroke).corners
 
         stroke.resample(32 * self.__turns)
         sv_nb = len(stroke) // self.__turns
@@ -818,13 +895,13 @@ class pyBluePrintCirclesShader(StrokeShader):
         R = self.__random_radius
         C = self.__random_center
 
-        # The directions (and phases) are calculated using a seperate
+        # The directions (and phases) are calculated using a separate
         # function decorated with an lru-cache. This guarantees that
         # the directions (involving sin and cos) are calculated as few
         # times as possible.
         #
         # This works because the phases and directions are only
-        # dependant on the stroke length, and the chance that
+        # dependent on the stroke length, and the chance that
         # stroke.resample() above produces strokes of the same length
         # is quite high.
         #
@@ -834,7 +911,7 @@ class pyBluePrintCirclesShader(StrokeShader):
 
         it = iter(stroke)
 
-        for j in range(self.__turns):
+        for _j in range(self.__turns):
             prev_radius = radius
             prev_center = center
             radius += randint(-R, R)
@@ -861,7 +938,7 @@ class pyBluePrintEllipsesShader(StrokeShader):
         self.__random_radius = random_radius
 
     def shade(self, stroke):
-        p_min, p_max = bounding_box(stroke)
+        p_min, p_max = BoundingBox.from_sequence(svert.point for svert in stroke).corners
 
         stroke.resample(32 * self.__turns)
         sv_nb = len(stroke) // self.__turns
@@ -875,7 +952,7 @@ class pyBluePrintEllipsesShader(StrokeShader):
         # for description of the line below, see pyBluePrintCirclesShader
         directions = phase_to_direction(sv_nb)
         it = iter(stroke)
-        for j in range(self.__turns):
+        for _j in range(self.__turns):
             prev_radius = radius
             prev_center = center
             radius = radius + Vector((randint(-R, R), randint(-R, R)))
@@ -886,7 +963,7 @@ class pyBluePrintEllipsesShader(StrokeShader):
                 c = prev_center + (center - prev_center) * phase
                 svert.point = (c.x + r.x * direction.x, c.y + r.y * direction.y)
 
-        # remove exessive vertices
+        # remove excess vertices
         if not it.is_end:
             it.increment()
             for sv in tuple(it):
@@ -898,7 +975,7 @@ class pyBluePrintEllipsesShader(StrokeShader):
 class pyBluePrintSquaresShader(StrokeShader):
     def __init__(self, turns=1, bb_len=10, bb_rand=0):
         StrokeShader.__init__(self)
-        self.__turns = turns # does not have any effect atm
+        self.__turns = turns  # does not have any effect atm
         self.__bb_len = bb_len
         self.__bb_rand = bb_rand
 
@@ -908,7 +985,7 @@ class pyBluePrintSquaresShader(StrokeShader):
             return
 
         # get minimum and maximum coordinates
-        p_min, p_max = bounding_box(stroke)
+        p_min, p_max = BoundingBox.from_sequence(svert.point for svert in stroke).corners
 
         stroke.resample(32 * self.__turns)
         num_segments = len(stroke) // self.__turns
@@ -927,7 +1004,7 @@ class pyBluePrintSquaresShader(StrokeShader):
             Vector((p_min.x - bb_len, p_max.y)),
             Vector((p_min.x, p_max.y + bb_len)),
             Vector((p_min.x, p_min.y - bb_len)),
-            )
+        )
 
         # add randomization to the points (if needed)
         if self.__bb_rand:
@@ -942,19 +1019,18 @@ class pyBluePrintSquaresShader(StrokeShader):
                 Vector((randint(-R, R), randint(-r, r))),
                 Vector((randint(-r, r), randint(-R, R))),
                 Vector((randint(-r, r), randint(-R, R))),
-                )
+            )
 
             # combine both tuples
             points = tuple(p + rand for (p, rand) in zip(points, randomization_mat))
 
-
-        # substract even from uneven; result is length four tuple of vectors
+        # subtract even from uneven; result is length four tuple of vectors
         it = iter(points)
         old_vecs = tuple(next(it) - current for current in it)
 
         it = iter(stroke)
         verticesToRemove = list()
-        for j in range(self.__turns):
+        for _j in range(self.__turns):
             for i, svert in zip(range(num_segments), it):
                 if i < first:
                     svert.point = points[0] + old_vecs[0] * i / (first - 1)
@@ -972,7 +1048,7 @@ class pyBluePrintSquaresShader(StrokeShader):
                     # special case; remove these vertices
                     verticesToRemove.append(svert)
 
-        # remove exessive vertices (if any)
+        # remove excess vertices (if any)
         if not it.is_end:
             it.increment()
             verticesToRemove += [svert for svert in it]
@@ -983,8 +1059,9 @@ class pyBluePrintSquaresShader(StrokeShader):
 
 class pyBluePrintDirectedSquaresShader(StrokeShader):
     """
-    Replaces the stroke with a directed square
+    Replaces the stroke with a directed square.
     """
+
     def __init__(self, turns=1, bb_len=10, mult=1):
         StrokeShader.__init__(self)
         self.__mult = mult
@@ -1010,16 +1087,19 @@ class pyBluePrintDirectedSquaresShader(StrokeShader):
 
         sqrt_coeff = sqrt(trace * trace - 4 * det)
         lambda1, lambda2 = (trace + sqrt_coeff) / 2, (trace - sqrt_coeff) / 2
-        # make sure those numers aren't to small, if they are, rooting them will yield complex numbers
+        # make sure those numbers aren't to small, if they are, rooting them will yield complex numbers
         lambda1, lambda2 = max(1e-12, lambda1), max(1e-12, lambda2)
         theta = atan(2 * p_var_xy / (p_var.x - p_var.y)) / 2
 
+        # Keep alignment for readability.
+        # autopep8: off
         if p_var.y > p_var.x:
             e1 = Vector((cos(theta + pi / 2), sin(theta + pi / 2))) * sqrt(lambda1) * self.__mult
             e2 = Vector((cos(theta + pi    ), sin(theta + pi    ))) * sqrt(lambda2) * self.__mult
         else:
             e1 = Vector((cos(theta), sin(theta)))                   * sqrt(lambda1) * self.__mult
             e2 = Vector((cos(theta + pi / 2), sin(theta + pi / 2))) * sqrt(lambda2) * self.__mult
+        # autopep8: on
 
         # partition the stroke
         num_segments = len(stroke) // self.__turns
@@ -1034,18 +1114,18 @@ class pyBluePrintDirectedSquaresShader(StrokeShader):
             p_mean - e1 * bb_len1 + e2,
             p_mean + e1 + e2 * bb_len2,
             p_mean + e1 * bb_len1 - e2,
-            )
+        )
 
         old_vecs = (
             e2 * bb_len2 * 2,
             e1 * bb_len1 * 2,
-           -e2 * bb_len2 * 2,
-           -e1 * bb_len1 * 2,
-            )
+            -e2 * bb_len2 * 2,
+            -e1 * bb_len1 * 2,
+        )
 
         it = iter(stroke)
         verticesToRemove = list()
-        for j in range(self.__turns):
+        for _j in range(self.__turns):
             for i, svert in zip(range(num_segments), it):
                 if i < first:
                     svert.point = points[0] + old_vecs[0] * i / (first - 1)
@@ -1063,7 +1143,7 @@ class pyBluePrintDirectedSquaresShader(StrokeShader):
                     # special case; remove these vertices
                     verticesToRemove.append(svert)
 
-        # remove exessive vertices
+        # remove excess vertices
         if not it.is_end:
             it.increment()
             verticesToRemove += [svert for svert in it]
@@ -1075,6 +1155,13 @@ class pyBluePrintDirectedSquaresShader(StrokeShader):
 # -- various (used in the parameter editor) -- #
 
 
+def iter_stroke_vertices(stroke, epsilon=1e-6):
+    yield stroke[0]
+    for prev, svert in pairwise(stroke):
+        if (prev.point - svert.point).length > epsilon:
+            yield svert
+
+
 class RoundCapShader(StrokeShader):
     def round_cap_thickness(self, x):
         x = max(0.0, min(x, 1.0))
@@ -1082,17 +1169,16 @@ class RoundCapShader(StrokeShader):
 
     def shade(self, stroke):
         # save the location and attribute of stroke vertices
-        buffer = tuple((Vector(sv.point), StrokeAttribute(sv.attribute)) for sv in stroke)
+        buffer = tuple((Vector(sv.point), StrokeAttribute(sv.attribute))
+                       for sv in iter_stroke_vertices(stroke))
         nverts = len(buffer)
         if nverts < 2:
             return
         # calculate the number of additional vertices to form caps
         thickness_beg = sum(stroke[0].attribute.thickness)
-        caplen_beg = thickness_beg / 2.0
         nverts_beg = max(5, int(thickness_beg))
 
         thickness_end = sum(stroke[-1].attribute.thickness)
-        caplen_end = (thickness_end) / 2.0
         nverts_end = max(5, int(thickness_end))
 
         # adjust the total number of stroke vertices
@@ -1104,7 +1190,7 @@ class RoundCapShader(StrokeShader):
         # reshape the cap at the beginning of the stroke
         q, attr = buffer[1]
         p, attr = buffer[0]
-        direction = (p - q).normalized() * caplen_beg
+        direction = (p - q).normalized() * thickness_beg
         n = 1.0 / nverts_beg
         R, L = attr.thickness
         for t, svert in zip(range(nverts_beg, 0, -1), stroke):
@@ -1115,7 +1201,7 @@ class RoundCapShader(StrokeShader):
         # reshape the cap at the end of the stroke
         q, attr = buffer[-2]
         p, attr = buffer[-1]
-        direction = (p - q).normalized() * caplen_beg
+        direction = (p - q).normalized() * thickness_end
         n = 1.0 / nverts_end
         R, L = attr.thickness
         for t, svert in zip(range(nverts_end, 0, -1), reversed(stroke)):
@@ -1130,7 +1216,8 @@ class RoundCapShader(StrokeShader):
 class SquareCapShader(StrokeShader):
     def shade(self, stroke):
         # save the location and attribute of stroke vertices
-        buffer = tuple((Vector(sv.point), StrokeAttribute(sv.attribute)) for sv in stroke)
+        buffer = tuple((Vector(sv.point), StrokeAttribute(sv.attribute))
+                       for sv in iter_stroke_vertices(stroke))
         nverts = len(buffer)
         if nverts < 2:
             return
