@@ -1,22 +1,4 @@
-# ##### BEGIN GPL LICENSE BLOCK #####
-#
-#  This program is free software; you can redistribute it and/or
-#  modify it under the terms of the GNU General Public License
-#  as published by the Free Software Foundation; either version 2
-#  of the License, or (at your option) any later version.
-#
-#  This program is distributed in the hope that it will be useful,
-#  but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#  GNU General Public License for more details.
-#
-#  You should have received a copy of the GNU General Public License
-#  along with this program; if not, write to the Free Software Foundation,
-#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
-#
-# ##### END GPL LICENSE BLOCK #####
-
-# <pep8-80 compliant>
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 __all__ = (
     "paths",
@@ -505,12 +487,7 @@ def disable_all():
 
 
 def _blender_manual_url_prefix():
-    if _bpy.app.version_cycle in {"rc", "release"}:
-        manual_version = "%d.%d" % _bpy.app.version[:2]
-    else:
-        manual_version = "dev"
-
-    return "https://docs.blender.org/manual/en/" + manual_version
+    return "https://docs.blender.org/manual/en/%d.%d" % _bpy.app.version[:2]
 
 
 def module_bl_info(mod, *, info_basis=None):
