@@ -1,28 +1,11 @@
-# ##### BEGIN GPL LICENSE BLOCK #####
-#
-#  This program is free software; you can redistribute it and/or
-#  modify it under the terms of the GNU General Public License
-#  as published by the Free Software Foundation; either version 2
-#  of the License, or (at your option) any later version.
-#
-#  This program is distributed in the hope that it will be useful,
-#  but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#  GNU General Public License for more details.
-#
-#  You should have received a copy of the GNU General Public License
-#  along with this program; if not, write to the Free Software Foundation,
-#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
-#
-# ##### END GPL LICENSE BLOCK #####
-
-# <pep8 compliant>
+# SPDX-License-Identifier: GPL-2.0-or-later
 import bpy
 from bpy.types import (
     Menu,
 )
 
 from bpy.app.translations import pgettext_tip as tip_
+from bpy.app.translations import pgettext_iface as iface_
 
 __all__ = (
     "ToolDef",
@@ -64,14 +47,14 @@ from collections import namedtuple
 ToolDef = namedtuple(
     "ToolDef",
     (
-        # Unique tool name (withing space & mode context).
+        # Unique tool name (within space & mode context).
         "idname",
         # The name to display in the interface.
         "label",
         # Description (for tool-tip), when not set, use the description of 'operator',
         # may be a string or a 'function(context, item, key-map) -> string'.
         "description",
-        # The name of the icon to use (found in ``release/datafiles/icons``) or None for no icon.
+        # The name of the icon to use (found in `release/datafiles/icons`) or None for no icon.
         "icon",
         # An optional cursor to use when this tool is active.
         "cursor",
@@ -81,12 +64,12 @@ ToolDef = namedtuple(
         "widget",
         # Optional key-map for tool, possible values are:
         #
-        # - ``None`` when the tool doesn't have a key-map.
+        # - `None` when the tool doesn't have a key-map.
         #   Also the default value when no key-map value is defined.
         #
         # - A string literal for the key-map name, the key-map items are located in the default key-map.
         #
-        # - ``()`` an empty tuple for a default name.
+        # - `()` an empty tuple for a default name.
         #   This is convenience functionality for generating a key-map name.
         #   So if a tool name is "Bone Size", in "Edit Armature" mode for the "3D View",
         #   All of these values are combined into an id, e.g:
@@ -98,7 +81,7 @@ ToolDef = namedtuple(
         # - A function that populates a key-maps passed in as an argument.
         #
         # - A tuple filled with triple's of:
-        #   ``(operator_id, operator_properties, keymap_item_args)``.
+        #   `(operator_id, operator_properties, keymap_item_args)`.
         #
         #   Use this to define the key-map in-line.
         #
@@ -106,7 +89,7 @@ ToolDef = namedtuple(
         #   Keep this functionality since it's likely useful for add-on key-maps.
         #
         # Warning: currently 'from_dict' this is a list of one item,
-        # so internally we can swap the key-map function for the key-map it's self.
+        # so internally we can swap the key-map function for the key-map itself.
         # This isn't very nice and may change, tool definitions shouldn't care about this.
         "keymap",
         # Optional data-block associated with this tool.
@@ -812,7 +795,7 @@ class ToolSelectPanelHelper:
         # Note: we could show 'item.text' here but it makes the layout jitter when switching tools.
         # Add some spacing since the icon is currently assuming regular small icon size.
         if show_tool_icon_always:
-            layout.label(text="    " + item.label, icon_value=icon_value)
+            layout.label(text="    " + iface_(item.label, "Operator"), icon_value=icon_value)
             layout.separator()
         else:
             if context.space_data.show_region_toolbar:
@@ -843,7 +826,7 @@ class ToolSelectPanelHelper:
             row.label(text="Drag:")
             row = split.row()
             row.context_pointer_set("tool", tool)
-            row.popover(panel="TOPBAR_PT_tool_fallback", text=label)
+            row.popover(panel="TOPBAR_PT_tool_fallback", text=iface_(label, "Operator"))
 
         return tool
 
@@ -1054,9 +1037,6 @@ def _activate_by_item(context, space_type, item, index, *, as_fallback=False):
         if props is None:
             print("Error:", gizmo_group, "could not access properties!")
         else:
-            for key in props.bl_rna.properties.keys():
-                props.property_unset(key)
-
             gizmo_properties = item.widget_properties
             if gizmo_properties is not None:
                 if not isinstance(gizmo_properties, list):
